"use strict";
function buildDescriptorsFromTree(root) {
    var pages = [];
    var folders = [];
    var pathTree = [];
    var filteredPathTree = [];
    buildPageFolderTree(root, pages, folders, pathTree, filteredPathTree, [], false);
    var folderContents = buildFolderContents(folders.map(function (f) { return f.path; }), pages.map(function (p) { return p.path; }));
    var filteredFolderContents = buildFolderContents(folders.filter(function (f) { return !f.hidden; }).map(function (f) { return f.path; }), pages.filter(function (p) { return !p.hidden; }).map(function (p) { return p.path; }));
    return { pages: pages, folders: folders, pathTree: pathTree, folderContents: folderContents, filteredPathTree: filteredPathTree, filteredFolderContents: filteredFolderContents };
}
exports.buildDescriptorsFromTree = buildDescriptorsFromTree;
function buildPageFolderTree(root, pages, folders, pathTree, filteredPathTree, chain, hidden) {
    pathTree.push(chain.join('/'));
    filteredPathTree.push(chain.join('/'));
    Object.keys(root).forEach(function (path) {
        var newChain = chain.concat(path);
        var absolutePath = newChain.join('/');
        if (root[path].contents) {
            var folder = root[path];
            folder.path = absolutePath;
            var folderHidden = hidden ? true : folder.hidden;
            var newPathTree = [];
            var newFilteredPathTree = [];
            pathTree.push(newPathTree);
            var folderDescriptor = {
                hidden: folderHidden,
                titleKey: folder.titleKey,
                path: absolutePath,
            };
            folders.push(folderDescriptor);
            buildPageFolderTree(folder.contents, pages, folders, newPathTree, newFilteredPathTree, newChain, folderHidden);
            if (newFilteredPathTree.length > 1 && !folderHidden) {
                filteredPathTree.push(newFilteredPathTree);
            }
            if (newFilteredPathTree.length <= 1) {
                folderDescriptor.hidden = true;
            }
        }
        else {
            var page = root[path];
            page.path = absolutePath;
            var pageHidden = hidden ? true : page.hidden;
            pathTree.push(absolutePath);
            if (!pageHidden) {
                filteredPathTree.push(absolutePath);
            }
            pages.push({
                titleKey: page.titleKey,
                hidden: pageHidden,
                component: page.component,
                reducerBuilder: page.reducerBuilder,
                persistent: page.persistent,
                path: absolutePath,
            });
        }
    });
}
function buildFolderContents(folders, pages) {
    var folderContents = {};
    folderContents[''] = pages.slice();
    folders.forEach(function (folder) {
        folderContents[folder] = [];
        pages.forEach(function (page) {
            if (page.indexOf(folder) !== -1) {
                folderContents[folder].push(page);
            }
        });
        folderContents[folder].sort();
    });
    return folderContents;
}
