"use strict";
var immutable_1 = require('immutable');
var persist_1 = require('./persist');
var actions_1 = require('./actions');
var DEISOLATE_KEY = '@@navigator_deisolate';
function deisolate(action) {
    action[DEISOLATE_KEY] = true;
    return action;
}
exports.deisolate = deisolate;
var Navigator = (function () {
    function Navigator(pages, prefix) {
        if (prefix === void 0) { prefix = ''; }
        this.instances = immutable_1.OrderedMap();
        this.activePageMutex = null;
        this.pages = pages;
        this.prefix = prefix;
    }
    Object.defineProperty(Navigator.prototype, "active", {
        get: function () {
            return this.instances.get(this.activePageMutex, null);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Navigator.prototype, "activeMutex", {
        get: function () {
            return this.activePageMutex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Navigator.prototype, "all", {
        get: function () {
            return this.instances;
        },
        enumerable: true,
        configurable: true
    });
    Navigator.prototype.reduce = function (action) {
        var _this = this;
        if (actions_1.NAVIGATION_TRANSIT_TO_PAGE.is(this.prefix, action)) {
            if (action.payload === null) {
                persist_1.saveActivePage(this.prefix, null);
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.instances = this.instances;
                return newNav;
            }
            if (this.instances.has(action.payload)) {
                if (this.instances.get(action.payload).descriptor.persistent !== false) {
                    persist_1.saveActivePage(this.prefix, action.payload);
                }
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.instances = this.instances;
                newNav.activePageMutex = action.payload;
                return newNav;
            }
            return this;
        }
        if (actions_1.NAVIGATION_OPEN_PAGE.is(this.prefix, action)) {
            if (this.instances.has(action.payload.mutex)) {
                if (this.instances.get(action.payload.mutex).descriptor.persistent !== false) {
                    persist_1.saveActivePage(this.prefix, action.payload.mutex);
                }
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.instances = this.instances;
                newNav.activePageMutex = action.payload.mutex;
                return newNav;
            }
            if (this.pages.hasPage(action.payload.path)) {
                var newPage = this.pages.buildInstance(action.payload);
                var newInstances = this.instances.set(action.payload.mutex, newPage);
                if (this.pages.getPage(action.payload.path).persistent !== false) {
                    persist_1.save(this.prefix, newInstances
                        .filter(function (i) { return i.descriptor.persistent !== false; })
                        .map(function (i) { return i.instantiationData; })
                        .toArray());
                    persist_1.saveActivePage(this.prefix, action.payload.mutex);
                }
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.instances = newInstances;
                newNav.activePageMutex = action.payload.mutex;
                return newNav;
            }
            return this;
        }
        if (actions_1.NAVIGATION_CLOSE_PAGE.is(this.prefix, action)) {
            var mutex = action.payload;
            if (this.instances.has(mutex)) {
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.activePageMutex = this.activePageMutex;
                if (this.activePageMutex === mutex) {
                    var ordering = this.instances.keySeq();
                    var index = ordering.indexOf(mutex);
                    var targetIndex = void 0;
                    if (index > 0) {
                        targetIndex = index - 1;
                    }
                    else {
                        targetIndex = 1;
                    }
                    var targetMutex = ordering.get(targetIndex, null);
                    if (this.instances.has(targetMutex) &&
                        this.instances.get(targetMutex).descriptor.persistent !== false) {
                        persist_1.saveActivePage(this.prefix, targetMutex);
                    }
                    else {
                        persist_1.saveActivePage(this.prefix, null);
                    }
                    newNav.activePageMutex = targetMutex;
                }
                newNav.instances = this.instances.remove(mutex);
                if (this.instances.get(action.payload).descriptor.persistent !== false) {
                    persist_1.save(this.prefix, newNav.instances
                        .filter(function (i) { return i.descriptor.persistent !== false; })
                        .map(function (i) { return i.instantiationData; })
                        .toArray());
                }
                return newNav;
            }
            return this;
        }
        if (actions_1.NAVIGATION_LOAD_PAGES.is(this.prefix, action)) {
            var instantiationData = persist_1.load(this.prefix);
            var loadedActivePageMutex_1 = persist_1.loadActivePage(this.prefix);
            var instances = void 0;
            var activePageMutex_1 = null;
            if (instantiationData && instantiationData.length > 0) {
                instances = immutable_1.OrderedMap(instantiationData.map(function (insData) {
                    if (_this.pages.hasPage(insData.path)) {
                        var pageInstance = _this.pages.buildInstance(insData);
                        if (pageInstance.mutex === loadedActivePageMutex_1) {
                            activePageMutex_1 = loadedActivePageMutex_1;
                        }
                        return [pageInstance.mutex, pageInstance];
                    }
                }).filter(function (i) { return !!i; }));
            }
            if (instances && instances.size > 0) {
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.instances = instances;
                newNav.activePageMutex = activePageMutex_1;
                return newNav;
            }
            return this;
        }
        if (actions_1.NAVIGATION_CLEAR_PAGES.is(this.prefix, action)) {
            persist_1.clear(this.prefix);
            return new Navigator(this.pages, this.prefix);
        }
        if (action[DEISOLATE_KEY]) {
            var modified_1 = false;
            var newInstances = this.instances.withMutations(function (i) {
                _this.instances.forEach(function (pageInstance) {
                    var newInstance = pageInstance.reduce(action);
                    if (newInstance !== pageInstance) {
                        modified_1 = true;
                        i.set(pageInstance.mutex, newInstance);
                    }
                });
            });
            if (modified_1) {
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.activePageMutex = this.activePageMutex;
                newNav.instances = newInstances;
                return newNav;
            }
            return this;
        }
        if (this.instances.has(this.activePageMutex)) {
            var target = this.instances.get(this.activePageMutex);
            var newTarget = target.reduce(action);
            if (target !== newTarget) {
                var newNav = new Navigator(this.pages, this.prefix);
                newNav.activePageMutex = this.activePageMutex;
                newNav.instances = this.instances.set(this.activePageMutex, newTarget);
                return newNav;
            }
            return this;
        }
        return this;
    };
    Navigator.prototype.transit = function (dispatch, mutex) {
        dispatch(actions_1.NAVIGATION_TRANSIT_TO_PAGE.create.unicast(this.prefix, mutex));
    };
    Navigator.prototype.open = function (dispatch, instantiationData) {
        dispatch(actions_1.NAVIGATION_OPEN_PAGE.create.unicast(this.prefix, instantiationData));
    };
    Navigator.prototype.close = function (dispatch, mutex) {
        dispatch(actions_1.NAVIGATION_CLOSE_PAGE.create.unicast(this.prefix, mutex));
    };
    Navigator.prototype.load = function (dispatch) {
        dispatch(actions_1.NAVIGATION_LOAD_PAGES.create.unicast(this.prefix));
    };
    Navigator.prototype.clear = function (dispatch) {
        dispatch(actions_1.NAVIGATION_CLEAR_PAGES.create.unicast(this.prefix));
    };
    return Navigator;
}());
exports.Navigator = Navigator;
