"use strict";
var PageInstance = (function () {
    function PageInstance(descriptor, instantiationData) {
        if (descriptor === null && instantiationData === null) {
            return;
        }
        var reducer = descriptor.reducerBuilder(instantiationData.reducerBuilderData);
        this.currentState = reducer(undefined, { type: '@@INIT_PAGE' });
        if (this.currentState === undefined) {
            throw new Error('The reducer did not return the initial state.');
        }
        this.params = {
            descriptor: descriptor,
            instantiationData: instantiationData,
            reducer: reducer,
            title: { key: descriptor.titleKey, data: instantiationData.titleData },
        };
    }
    Object.defineProperty(PageInstance.prototype, "mutex", {
        get: function () {
            return this.params.instantiationData.mutex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PageInstance.prototype, "path", {
        get: function () {
            return this.params.descriptor.path;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PageInstance.prototype, "title", {
        get: function () {
            return this.params.title;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PageInstance.prototype, "state", {
        get: function () {
            return this.currentState;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PageInstance.prototype, "descriptor", {
        get: function () {
            return this.params.descriptor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PageInstance.prototype, "instantiationData", {
        get: function () {
            return this.params.instantiationData;
        },
        enumerable: true,
        configurable: true
    });
    PageInstance.prototype.reduce = function (action) {
        var newState = this.params.reducer(this.state, action);
        if (newState === this.state) {
            return this;
        }
        var clone = new PageInstance(null, null);
        clone.params = this.params;
        clone.currentState = newState;
        return clone;
    };
    return PageInstance;
}());
exports.PageInstance = PageInstance;
