"use strict";
var immutable_1 = require('immutable');
var pageInstance_1 = require('./pageInstance');
var buildDescriptorsFromTree_1 = require('./buildDescriptorsFromTree');
;
var Pages = (function () {
    function Pages(root) {
        var descriptors = buildDescriptorsFromTree_1.buildDescriptorsFromTree(root);
        this.pages = immutable_1.Map(descriptors.pages.map(function (d) { return [d.path, d]; }));
        this.folders = immutable_1.Map(descriptors.folders.map(function (d) { return [d.path, d]; }));
        this.calculatedPathTree = descriptors.pathTree;
        this.calculatedFolderContents = descriptors.folderContents;
        this.calculatedFilteredPathTree = descriptors.filteredPathTree;
        this.calculatedFilteredFolderContents = descriptors.filteredFolderContents;
    }
    Pages.prototype.hasPage = function (path) {
        return this.pages.has(path);
    };
    Pages.prototype.getPage = function (path) {
        return this.pages.get(path);
    };
    Pages.prototype.hasFolder = function (path) {
        return this.folders.has(path);
    };
    Pages.prototype.getFolder = function (path) {
        return this.folders.get(path);
    };
    Pages.prototype.isHidden = function (path) {
        if (this.hasPage(path)) {
            return !!this.getPage(path).hidden;
        }
        if (this.getFolder(path)) {
            return !!this.getFolder(path).hidden;
        }
        return null;
    };
    Object.defineProperty(Pages.prototype, "pathTree", {
        get: function () {
            return this.calculatedPathTree;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pages.prototype, "folderContents", {
        get: function () {
            return this.calculatedFolderContents;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pages.prototype, "filteredPathTree", {
        get: function () {
            return this.calculatedFilteredPathTree;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pages.prototype, "filteredFolderContents", {
        get: function () {
            return this.calculatedFilteredFolderContents;
        },
        enumerable: true,
        configurable: true
    });
    Pages.prototype.buildInstance = function (instantiationData) {
        if (!this.hasPage(instantiationData.path)) {
            throw new Error("No descriptor exists for the provided path (" + instantiationData.path + ")");
        }
        return new pageInstance_1.PageInstance(this.pages.get(instantiationData.path), instantiationData);
    };
    return Pages;
}());
exports.Pages = Pages;
